/*
 * PokeFrog - A 'Pokemonized' verison of Frogger, the well-known Konami game.
 *
 * copyright (c) 2012 Fabien LOISON <http://www.flogisoft.com/>
 * 
 * this program is free software. it comes without any warranty, to
 * the extent permitted by applicable law. you can redistribute it
 * and/or modify it under the terms of the do what the fuck you want
 * to public license, version 2, as published by sam hocevar. see
 * http://sam.zoy.org/wtfpl/copying for more details. 
 */


/**
 * @module PokeFrog
 */


/**
 * The player.
 *
 * @class Player
 * @constructor
 *
 * @param {wjs.Tileset} tileset The tileset that contain the player's sprites.
 *        The following tiles must be available: `up`, `up-walk`, `right`,
 *        `right-walk`, `down`, `down-walk`, `left` and `left-walk`.
 * @param {Array} hitBox The hitbox of the player (relative to the player's
 *        tile): `[x1, y1, x2, y2]`.
 */
function Player(tileset, hitbox)
{
	this._tileset = tileset;
	this._hitbox = hitbox;

	this._x = 425;
	this._y = 386;
	this._orientation = 'up';
	this._isWalking = false;
	this._visible = true;
	this._speed = 3;
	this._blink = 100;

	// Register events
	wjs.kbEvent.addKeyDownCallback('player', this._onKeyDown.bind(this));
	wjs.kbEvent.addKeyUpCallback('player', this._onKeyUp.bind(this));
}

/**
 * Calculate the position of the player's hitbox on the canvas.
 *
 * @method getHitbox
 *
 * @return {Array} The hitbox: `[x1, y1, x2, y2]`.
 */
Player.prototype.getHitbox = function()
{
	return [
		this._x + this._hitbox[0],  // x1
		this._y + this._hitbox[1],  // y1
		this._x + this._hitbox[2],  // x2
		this._y + this._hitbox[3]]  // y2
}

/**
 * Update the state of the player.
 *
 * @method update
 */
Player.prototype.update = function()
{
	var hitbox = this.getHitbox();

	// Move the player
	if (this._isWalking && this._blink <= 0)
	{
		switch (this._orientation)
		{
			case 'left':
				this._x -= this._speed;
				break;
			case 'right':
				this._x += this._speed;
				break;
			case 'up':
				this._y -= this._speed;
				break;
			case 'down':
				this._y += this._speed;
				break;
		}
	}

	// Blink
	if (this._blink > 0)
	{
		if (this._blink % 10 == 0)
		{
			this._visible = !this._visible;
		}
		this._blink--;
	}
	else
	{
		this._visible = true;
	}

	// Avoid player to go out of the screen
	if (hitbox[0] <= 0)
	{
		this._x -= hitbox[0]-1;
	}
	if (hitbox[2] >= wjs.canvas.width)
	{
		this._x -= hitbox[2] - wjs.canvas.width+1;
	}
	if (hitbox[1] <= 0)
	{
		this._y -= hitbox[1]-1;
	}
	if (hitbox[3] >= wjs.canvas.height)
	{
		this._y -= hitbox[3] - wjs.canvas.height+1;
	}
}

/**
 * Draw the player on the canvas.
 *
 * @method draw
 */
Player.prototype.draw = function()
{
	if (this._visible)
	{
		var tileId = this._orientation;
		if (this._isWalking)
		{
			tileId += '-walk';
		}
		this._tileset.blit(tileId, this._x, this._y);
	}

	// Debug: display the hitbox
	if (DEBUG)
	{
		var ctx = wjs.canvas.getContext('2d');
		var hitbox = this.getHitbox();
		ctx.lineWidth = 2;
		ctx.strokeStyle = '#f00';
		ctx.strokeRect(
				hitbox[0],  // x
				hitbox[1],  // y
				hitbox[2]-hitbox[0],  // width
				hitbox[3]-hitbox[1]); // height
	}
}

/**
 * Kill the player (unregister events,...)
 *
 * @method die
 */
Player.prototype.die = function()
{
	this._visible = false;
	wjs.kbEvent.delKeyDownCallback('player');
	wjs.kbEvent.delKeyUpCallback('player');
}

/**
 * Callback for the `keydown` event.
 *
 * @method _onKeyDown
 * @private
 *
 * @param {String} keyName The pressed key.
 */
Player.prototype._onKeyDown = function(keyName)
{
	if (keyName == 'left' || keyName == 'up' || keyName == 'right' || keyName == 'down')
	{
		this._orientation = keyName;
		this._isWalking = true;
	}
}

/**
 * Callback for the `keyup` event.
 *
 * @method _onKeyUp
 * @private
 *
 * @param {String} keyName The pressed key.
 */
Player.prototype._onKeyUp = function(keyName)
{
	if (keyName == this._orientation)
	{
		this._isWalking = false;
	}
}
